@@ -17,6 +17,7 @@
 #![allow(clippy::too_many_arguments, clippy::blocks_in_if_conditions)]
 
 use std::fmt::Write;
+use std::fs::File;
 use std::io::{stdin, BufRead};
 use std::time::SystemTime;
 
@@ -24,10 +25,13 @@ use eyre::Context;
 use git_branchless_invoke::CommandContext;
 use git_branchless_opts::{HookArgs, HookSubcommand};
 use itertools::Itertools;
-use lib::util::ExitCode;
+use lib::core::dag::Dag;
+use lib::core::repo_ext::RepoExt;
+use lib::core::rewrite::rewrite_hooks::get_deferred_commits_path;
+use lib::util::EyreExitOr;
 use tracing::{error, instrument, warn};
 
-use lib::core::eventlog::{should_ignore_ref_updates, Event, EventLogDb};
+use lib::core::eventlog::{should_ignore_ref_updates, Event, EventLogDb, EventReplayer};
 use lib::core::formatting::{Glyphs, Pluralize};
 use lib::core::gc::{gc, mark_commit_reachable};
 use lib::git::{CategorizedReferenceName, MaybeZeroOid, NonZeroOid, ReferenceName, Repo};
@@ -102,6 +106,32 @@ fn hook_post_commit_common(effects: &Effects, hook_name: &str) -> eyre::Result<(
     mark_commit_reachable(&repo, commit_oid)
         .wrap_err("Marking commit as reachable for GC purposes")?;
 
+    let event_replayer = EventReplayer::from_event_log_db(effects, &repo, &event_log_db)?;
+    let event_cursor = event_replayer.make_default_cursor();
+    let references_snapshot = repo.get_references_snapshot()?;
+    Dag::open_and_sync(
+        effects,
+        &repo,
+        &event_replayer,
+        event_cursor,
+        &references_snapshot,
+    )?;
+
+    if repo.is_rebase_underway()? {
+        let deferred_commits_path = get_deferred_commits_path(&repo);
+        let mut deferred_commits_file = File::options()
+            .create(true)
+            .append(true)
+            .open(&deferred_commits_path)
+            .with_context(|| {
+                format!("Opening deferred commits file at {deferred_commits_path:?}")
+            })?;
+
+        use std::io::Write;
+        writeln!(deferred_commits_file, "{commit_oid}")?;
+        return Ok(());
+    }
+
     let timestamp = commit.get_time().to_system_time()?;
 
     // Potentially lossy conversion. The semantics are to round to the nearest
@@ -176,8 +206,12 @@ mod reference_transaction {
             // The leading `# pack-refs with:` pragma.
             return None;
         }
+        if line.starts_with('^') {
+            // A peeled ref
+            // FIXME actually support peeled refs in packed-refs
+            return None;
+        }
         if !line.starts_with(|c: char| c.is_ascii_hexdigit()) {
-            // The leading `# pack-refs with:` pragma.
             warn!(?line, "Unrecognized pack-refs line starting character");
             return None;
         }
@@ -488,7 +522,7 @@ fn hook_reference_transaction(effects: &Effects, transaction_state: &str) -> eyr
 
 /// `hook` subcommand.
 #[instrument]
-pub fn command_main(ctx: CommandContext, args: HookArgs) -> eyre::Result<ExitCode> {
+pub fn command_main(ctx: CommandContext, args: HookArgs) -> EyreExitOr<()> {
     let CommandContext {
         effects,
         git_run_info,
@@ -548,5 +582,5 @@ pub fn command_main(ctx: CommandContext, args: HookArgs) -> eyre::Result<ExitCod
         }
     }
 
-    Ok(ExitCode(0))
+    Ok(Ok(()))
 }
