@@ -30,7 +30,7 @@ use lib::core::effects::Effects;
 use lib::core::formatting::Glyphs;
 use lib::git::GitRunInfo;
 use lib::git::{Repo, RepoError};
-use lib::util::ExitCode;
+use lib::util::{ExitCode, EyreExitOr};
 use tracing::level_filters::LevelFilter;
 use tracing::{info, instrument, warn};
 use tracing_chrome::ChromeLayerBuilder;
@@ -166,17 +166,17 @@ support repositories with this configuration option enabled.",
 /// call `invoke_subcommand_main` instead.
 #[instrument(skip(f))]
 pub fn do_main_and_drop_locals<T: Parser>(
-    f: impl Fn(CommandContext, T) -> eyre::Result<ExitCode>,
+    f: impl Fn(CommandContext, T) -> EyreExitOr<()>,
     args: Vec<OsString>,
 ) -> eyre::Result<i32> {
     let command = GlobalArgs::command();
+    let command_args = T::parse_from(&args);
     let matches = command.ignore_errors(true).get_matches_from(&args);
     let GlobalArgs {
         working_directory,
         color,
     } = GlobalArgs::from_arg_matches(&matches)
         .map_err(|err| eyre::eyre!("Could not parse global arguments: {err}"))?;
-    let command_args = T::parse_from(args);
 
     if let Some(working_directory) = working_directory {
         std::env::set_current_dir(&working_directory).wrap_err_with(|| {
@@ -214,8 +214,13 @@ pub fn do_main_and_drop_locals<T: Parser>(
         effects,
         git_run_info,
     };
-    let ExitCode(exit_code) = f(ctx, command_args)?;
-    let exit_code: i32 = exit_code.try_into()?;
+    let exit_code = match f(ctx, command_args)? {
+        Ok(()) => 0,
+        Err(ExitCode(exit_code)) => {
+            let exit_code: i32 = exit_code.try_into()?;
+            exit_code
+        }
+    };
     Ok(exit_code)
 }
 
@@ -228,7 +233,7 @@ pub fn do_main_and_drop_locals<T: Parser>(
 /// }
 /// ```
 #[instrument(skip(f))]
-pub fn invoke_subcommand_main<T: Parser>(f: impl Fn(CommandContext, T) -> eyre::Result<ExitCode>) {
+pub fn invoke_subcommand_main<T: Parser>(f: impl Fn(CommandContext, T) -> EyreExitOr<()>) {
     // Install panic handler.
     color_eyre::install().expect("Could not install panic handler");
     let args = std::env::args_os().collect();
