@@ -24,7 +24,7 @@ fn nth_parent(commit: Commit, n: usize) -> Commit {
     for _i in 0..n {
         commit = match commit.get_parents().first() {
             Some(commit) => commit.clone(),
-            None => panic!("Couldn't find parent of: {:?}", commit),
+            None => panic!("Couldn't find parent of: {commit:?}"),
         }
     }
     commit
@@ -64,7 +64,7 @@ fn bench_rebase_plan(c: &mut Criterion) {
         };
         let permissions = RebasePlanPermissions::verify_rewrite_set(
             &dag,
-            &build_options,
+            build_options,
             &CommitSet::from(later_commit.get_oid()),
         )
         .unwrap()
