@@ -1,7 +1,6 @@
 //! Wrapper around the Eden SCM directed acyclic graph implementation, which
 //! allows for efficient graph queries.
 
-use std::borrow::Borrow;
 use std::cmp::Ordering;
 use std::collections::HashMap;
 use std::convert::TryFrom;
@@ -268,7 +267,7 @@ impl Dag {
         futures::executor::block_on(fut)
     }
 
-    /// This function's code adapted from `GitDag`, licensed under GPL-2.
+    /// Update the DAG with all commits reachable from branches.
     #[instrument]
     fn sync(&mut self, effects: &Effects, repo: &Repo) -> eyre::Result<()> {
         let master_heads = self.main_branch_commit.clone();
@@ -358,9 +357,13 @@ impl Dag {
 
     /// Wrapper around DAG method.
     #[instrument]
-    pub fn sort(&self, commit_set: &CommitSet) -> eyre::Result<CommitSet> {
-        let result = self.run_blocking(self.inner.sort(commit_set))?;
-        Ok(result)
+    pub fn sort(&self, commit_set: &CommitSet) -> eyre::Result<Vec<NonZeroOid>> {
+        let commit_set = self.run_blocking(self.inner.sort(commit_set))?;
+        let commit_oids = self.commit_set_to_vec(&commit_set)?;
+
+        // `.sort` seems to sort it such that the child-most commits are first?
+        // In all current use-cases, we want to start with the parent commits.
+        Ok(commit_oids.into_iter().rev().collect())
     }
 
     /// Eagerly convert a `CommitSet` into a `Vec<NonZeroOid>` by iterating over it, preserving order.
@@ -395,7 +398,7 @@ impl Dag {
 
     /// Conduct an arbitrary query against the DAG.
     pub fn query(&self) -> &eden_dag::Dag {
-        self.inner.borrow()
+        &self.inner
     }
 
     /// Determine whether or not the given commit is a public commit (i.e. is an
