@@ -35,18 +35,19 @@ pub enum OperationType {
     InitializeRebase,
     MakeGraph,
     ProcessEvents,
-    PushBranches,
+    PushCommits,
     QueryWorkingCopy,
     ReadingFromCache,
     RebaseCommits,
     RepairBranches,
     RepairCommits,
     RunGitCommand(Arc<String>),
-    RunTests(Arc<String>),
     RunTestOnCommit(Arc<String>),
+    RunTests(Arc<String>),
     SortCommits,
     SyncCommits,
     UpdateCommitGraph,
+    UpdateCommits,
     WalkCommits,
 }
 
@@ -70,7 +71,7 @@ impl ToString for OperationType {
             OperationType::GetUpstreamPatchIds => "Enumerating patch IDs",
             OperationType::InitializeRebase => "Initializing rebase",
             OperationType::MakeGraph => "Examining local history",
-            OperationType::PushBranches => "Pushing branches",
+            OperationType::PushCommits => "Pushing branches",
             OperationType::ProcessEvents => "Processing events",
             OperationType::QueryWorkingCopy => "Querying the working copy",
             OperationType::ReadingFromCache => "Reading from cache",
@@ -80,10 +81,11 @@ impl ToString for OperationType {
             OperationType::RunGitCommand(command) => {
                 return format!("Running Git command: {}", &command)
             }
-            OperationType::RunTests(command) => return format!("Running tests: {command}"),
-            OperationType::RunTestOnCommit(commit) => return format!("Waiting to test {commit}"),
+            OperationType::RunTests(command) => return format!("Running command: {command}"),
+            OperationType::RunTestOnCommit(commit) => return format!("Waiting to run on {commit}"),
             OperationType::SortCommits => "Sorting commits",
             OperationType::SyncCommits => "Syncing commit stacks",
+            OperationType::UpdateCommits => "Updating commits",
             OperationType::UpdateCommitGraph => "Updating commit graph",
             OperationType::WalkCommits => "Walking commits",
         };
