@@ -3,7 +3,7 @@ use std::convert::TryInto;
 use std::ffi::{OsStr, OsString};
 use std::fmt::Write;
 use std::io::{BufRead, BufReader, Read, Write as WriteIo};
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus, Stdio};
 use std::sync::Arc;
 use std::thread::{self, JoinHandle};
@@ -17,7 +17,7 @@ use crate::core::config::get_hooks_dir;
 use crate::core::effects::{Effects, OperationType};
 use crate::core::eventlog::{EventTransactionId, BRANCHLESS_TRANSACTION_ID_ENV_VAR};
 use crate::git::repo::Repo;
-use crate::util::{get_sh, ExitCode};
+use crate::util::{get_sh, ExitCode, EyreExitOr};
 
 /// Path to the `git` executable on disk to be executed.
 #[derive(Clone)]
@@ -113,7 +113,7 @@ impl GitRunInfo {
         effects: &Effects,
         event_tx_id: Option<EventTransactionId>,
         args: &[&OsStr],
-    ) -> eyre::Result<ExitCode> {
+    ) -> EyreExitOr<()> {
         let GitRunInfo {
             path_to_git,
             working_directory,
@@ -167,7 +167,11 @@ impl GitRunInfo {
             .try_into()
             .wrap_err("Converting exit code from i32 to isize")?;
         let exit_code = ExitCode(exit_code);
-        Ok(exit_code)
+        if exit_code.is_success() {
+            Ok(Ok(()))
+        } else {
+            Ok(Err(exit_code))
+        }
     }
 
     /// Run Git in a subprocess, and inform the user.
@@ -186,7 +190,7 @@ impl GitRunInfo {
         effects: &Effects,
         event_tx_id: Option<EventTransactionId>,
         args: &[S],
-    ) -> eyre::Result<ExitCode> {
+    ) -> EyreExitOr<()> {
         self.run_inner(
             effects,
             event_tx_id,
@@ -203,7 +207,7 @@ impl GitRunInfo {
         &self,
         event_tx_id: Option<EventTransactionId>,
         args: &[impl AsRef<OsStr> + std::fmt::Debug],
-    ) -> eyre::Result<ExitCode> {
+    ) -> EyreExitOr<()> {
         let GitRunInfo {
             path_to_git,
             working_directory,
@@ -232,7 +236,51 @@ impl GitRunInfo {
             .try_into()
             .wrap_err("Converting exit code from i32 to isize")?;
         let exit_code = ExitCode(exit_code);
-        Ok(exit_code)
+        if exit_code.is_success() {
+            Ok(Ok(()))
+        } else {
+            Ok(Err(exit_code))
+        }
+    }
+
+    /// Returns the working directory for commands run on a given `Repo`.
+    ///
+    /// This is typically the working copy path for the repo.
+    ///
+    /// Some commands (notably `git status`) do not function correctly when run
+    /// from the git repo (i.e. `.git`) path.
+    /// Hooks should also be run from the working copy path - from
+    /// `githooks(5)`: Before Git invokes a hook, it changes its working
+    /// directory to either $GIT_DIR in a bare repository or the root of the
+    /// working tree in a non-bare repository.
+    ///
+    /// This contains additional logic for symlinked `.git` directories to work
+    /// around an upstream `libgit2` issue.
+    fn working_directory<'a>(&'a self, repo: &'a Repo) -> &'a Path {
+        repo.get_working_copy_path()
+            // `libgit2` returns the working copy path as the parent of the
+            // `.git` directory. However, if the `.git` directory is a symlink,
+            // `libgit2` *resolves the symlink*, returning an incorrect working
+            // directory: https://github.com/libgit2/libgit2/issues/6401
+            //
+            // Detect this case by checking if the current working directory is
+            // not a subdirectory of the returned working copy. While this does
+            // not work if the symlinked `.git` directory points to a child of
+            // an ancestor of the directory, this should handle most cases,
+            // including working copies managed by the `repo` tool.
+            //
+            // This workaround may result in slightly incorrect hook behavior,
+            // as the current working directory may be a subdirectory of the
+            // root of the working tree.
+            .map(|working_copy| {
+                // Both paths are already "canonicalized".
+                if !self.working_directory.starts_with(working_copy) {
+                    &self.working_directory
+                } else {
+                    working_copy
+                }
+            })
+            .unwrap_or_else(|| repo.get_path())
     }
 
     fn run_silent_inner(
@@ -252,12 +300,7 @@ impl GitRunInfo {
             stdin,
         } = opts;
 
-        // Prefer running in the working copy path to the repo path, because
-        // some commands (notably `git status`) to not function correctly
-        // when run from the git repo (i.e. `.git`) path.
-        let repo_path = repo
-            .get_working_copy_path()
-            .unwrap_or_else(|| repo.get_path());
+        let repo_path = self.working_directory(repo);
         // Technically speaking, we should be able to work with non-UTF-8 repository
         // paths. Need to make the typechecker accept it.
         let repo_path = repo_path.to_str().ok_or_else(|| {
@@ -375,13 +418,7 @@ impl GitRunInfo {
 
         if hook_dir.join(hook_name).exists() {
             let mut child = Command::new(get_sh().ok_or_else(|| eyre!("could not get sh"))?)
-                // From `githooks(5)`: Before Git invokes a hook, it changes its
-                // working directory to either $GIT_DIR in a bare repository or the
-                // root of the working tree in a non-bare repository.
-                .current_dir(
-                    repo.get_working_copy_path()
-                        .unwrap_or_else(|| repo.get_path()),
-                )
+                .current_dir(self.working_directory(repo))
                 .arg("-c")
                 .arg(format!("{hook_name} \"$@\""))
                 .arg(hook_name) // "$@" expands "$1" "$2" "$3" ... but we also must specify $0.
