@@ -17,7 +17,6 @@ use crate::core::config::env_vars::{
 };
 use crate::git::{GitRunInfo, GitVersion, NonZeroOid, Repo};
 use crate::util::get_sh;
-
 use color_eyre::Help;
 use eyre::Context;
 use itertools::Itertools;
@@ -141,17 +140,14 @@ impl Git {
             .expect("Unable to find git-branchless path parent");
         let bash = get_sh().expect("bash missing?");
         let bash_path = bash.parent().unwrap();
-        std::env::join_paths(
-            vec![
-                // For Git to be able to launch `git-branchless`.
-                branchless_path.as_os_str(),
-                // For our hooks to be able to call back into `git`.
-                self.git_exec_path.as_os_str(),
-                // For branchless to manually invoke bash when needed.
-                bash_path.as_os_str(),
-            ]
-            .into_iter(),
-        )
+        std::env::join_paths(vec![
+            // For Git to be able to launch `git-branchless`.
+            branchless_path.as_os_str(),
+            // For our hooks to be able to call back into `git`.
+            self.git_exec_path.as_os_str(),
+            // For branchless to manually invoke bash when needed.
+            bash_path.as_os_str(),
+        ])
         .expect("joining paths")
     }
 
@@ -341,20 +337,45 @@ stderr:
         let result = self.run_with_options(&git_run_args, options);
 
         if !should_use_separate_command_binary(subcommand) {
-            result
+            let main_command_exe = assert_cmd::cargo::cargo_bin("git-branchless");
+            let subcommand_exe =
+                assert_cmd::cargo::cargo_bin(format!("git-branchless-{subcommand}"));
+            if main_command_exe.exists() && subcommand_exe.exists() {
+                let main_command_mtime = main_command_exe.metadata()?.modified()?;
+                let subcommand_mtime = subcommand_exe.metadata()?.modified()?;
+                if subcommand_mtime > main_command_mtime {
+                    result.suggestion(format!(
+                        "\
+The modified time for {main_command_exe:?} was before the modified time for
+{subcommand_exe:?}, which may indicate that you made changes to the subcommand
+without building the main executable. This may cause spurious test failures
+because the main executable code is out of date.
+
+If so, you should either explicitly run: cargo -p git-branchless
+to build the main executable before running this test; or, if it's okay to skip
+building the main executable and test only the subcommand executable, you
+can set the environment variable
+`{TEST_SEPARATE_COMMAND_BINARIES}={subcommand}` to directly invoke it.\
+"
+                    ))
+                } else {
+                    result
+                }
+            } else {
+                result
+            }
         } else {
             result.suggestion(format!(
                 "\
-If you have set the TEST_SEPARATE_COMMAND_BINARIES environment variable, then \
+If you have set the {TEST_SEPARATE_COMMAND_BINARIES} environment variable, then \
 the git-branchless-{subcommand} binary is NOT automatically built or updated when \
 running integration tests for other binaries (see \
 https://github.com/rust-lang/cargo/issues/4316 for more details).
 
 Make sure that git-branchless-{subcommand} has been built before running \
-integration tests. You can build it with: cargo build -p
-git-branchless-{subcommand}
+integration tests. You can build it with: cargo build -p git-branchless-{subcommand}
 
-If you have not set the TEST_SEPARATE_COMMAND_BINARIES environment variable, \
+If you have not set the {TEST_SEPARATE_COMMAND_BINARIES} environment variable, \
 then you can only run tests in the main `git-branchless` and \
 `git-branchless-lib` crates.\
         ",
@@ -369,6 +390,7 @@ then you can only run tests in the main `git-branchless` and \
         self.run(&["init"])?;
         self.run(&["config", "user.name", DUMMY_NAME])?;
         self.run(&["config", "user.email", DUMMY_EMAIL])?;
+        self.run(&["config", "core.abbrev", "7"])?;
 
         if options.make_initial_commit {
             self.commit_file("initial", 0)?;
@@ -460,19 +482,34 @@ then you can only run tests in the main `git-branchless` and \
         Ok(())
     }
 
-    /// Commit a file with default contents. The `time` argument is used to set
-    /// the commit timestamp, which is factored into the commit hash.
+    /// Get a diff of a commit with the a/b file header removed. Only works for commits
+    /// with a single file.
     #[instrument]
-    pub fn commit_file_with_contents(
+    pub fn get_trimmed_diff(&self, file: &str, commit: &str) -> eyre::Result<String> {
+        let (stdout, _stderr) = self.run(&["show", "--pretty=format:", commit])?;
+        let split_on = format!("+++ b/{file}\n");
+        match stdout.as_str().split_once(split_on.as_str()) {
+            Some((_, diff)) => Ok(diff.to_string()),
+            None => eyre::bail!("Error trimming diff. Could not split on '{split_on}'"),
+        }
+    }
+
+    /// Commit a file with given contents and message. The `time` argument is
+    /// used to set the commit timestamp, which is factored into the commit
+    /// hash. The filename is always appended to the message prefix.
+    #[instrument]
+    pub fn commit_file_with_contents_and_message(
         &self,
         name: &str,
         time: isize,
         contents: &str,
+        message_prefix: &str,
     ) -> eyre::Result<NonZeroOid> {
+        let message = format!("{message_prefix} {name}.txt");
         self.write_file_txt(name, contents)?;
         self.run(&["add", "."])?;
         self.run_with_options(
-            &["commit", "-m", &format!("create {name}.txt")],
+            &["commit", "-m", &message],
             &GitRunOptions {
                 time,
                 ..Default::default()
@@ -487,6 +524,19 @@ then you can only run tests in the main `git-branchless` and \
         Ok(oid)
     }
 
+    /// Commit a file with given contents and a default message. The `time`
+    /// argument is used to set the commit timestamp, which is factored into the
+    /// commit hash.
+    #[instrument]
+    pub fn commit_file_with_contents(
+        &self,
+        name: &str,
+        time: isize,
+        contents: &str,
+    ) -> eyre::Result<NonZeroOid> {
+        self.commit_file_with_contents_and_message(name, time, contents, "create")
+    }
+
     /// Commit a file with default contents. The `time` argument is used to set
     /// the commit timestamp, which is factored into the commit hash.
     pub fn commit_file(&self, name: &str, time: isize) -> eyre::Result<NonZeroOid> {
@@ -746,8 +796,12 @@ pub fn remove_rebase_lines(output: String) -> String {
             // to deal with multiple possible output formats, just remove the
             // line in question.
             !line.contains("Auto-merging")
+            // Message changed between Git versions (due to be included in Git
+            // v2.42) in
+            // https://github.com/git/git/commit/d92304ff5cfdca463e9ecd1345807d0b46d6af33.
+            && !line.contains("use \"git pull\"")
         })
-        .map(|line| format!("{line}\n"))
+        .flat_map(|line| [line, "\n"])
         .collect()
 }
 
@@ -759,19 +813,48 @@ pub fn trim_lines(output: String) -> String {
         .collect()
 }
 
+/// Remove lines which are not present or different between Git versions.
+pub fn remove_nondeterministic_lines(output: String) -> String {
+    output
+        .lines()
+        .filter(|line| {
+            // This line is not present in some Git versions.
+            !line.contains("Fetching")
+                // This line is produced in a different order in some Git versions.
+                && !line.contains("Your branch is up to date")
+                // This line is only sometimes produced in CI for some reason? I
+                // don't understand how it would only sometimes print this
+                // message, but it does.
+                && !line.contains("Switched to branch")
+                // Hint text is more likely to change between Git versions.
+                && !line.contains("hint:")
+                // There are weird non-deterministic failures for test
+                // `test_sync_no_delete_main_branch` where an extra newline is
+                // printed, such as in
+                // https://github.com/arxanas/git-branchless/actions/runs/5609690113/jobs/10263760651?pr=1002
+                && !line.is_empty()
+        })
+        .flat_map(|line| [line, "\n"])
+        .collect()
+}
+
 /// Utilities for testing in a virtual terminal (PTY).
 pub mod pty {
-    use std::{
-        sync::{mpsc::channel, Arc, Mutex},
-        thread,
-        time::Duration,
-    };
+    use std::sync::{mpsc::channel, Arc, Mutex};
+    use std::thread;
+    use std::time::Duration;
 
     use eyre::eyre;
-    use portable_pty::{native_pty_system, CommandBuilder, PtySize};
+    use portable_pty::{native_pty_system, CommandBuilder, ExitStatus, PtySize};
 
     use super::Git;
 
+    /// Terminal escape code corresponding to pressing the up arrow key.
+    pub const UP_ARROW: &str = "\x1b[A";
+
+    /// Terminal escape code corresponding to pressing the down arrow key.
+    pub const DOWN_ARROW: &str = "\x1b[B";
+
     /// An action to take as part of the PTY test script.
     pub enum PtyAction<'a> {
         /// Input the provided string as keystrokes to the terminal.
@@ -783,18 +866,23 @@ pub mod pty {
     }
 
     /// Run the provided script in the context of a virtual terminal.
+    #[track_caller]
     pub fn run_in_pty(
         git: &Git,
         branchless_subcommand: &str,
         args: &[&str],
         inputs: &[PtyAction],
-    ) -> eyre::Result<()> {
+    ) -> eyre::Result<ExitStatus> {
         // Use the native pty implementation for the system
         let pty_system = native_pty_system();
         let pty_size = PtySize::default();
-        let mut pty = pty_system
+        let pty = pty_system
             .openpty(pty_size)
             .map_err(|e| eyre!("Could not open pty: {}", e))?;
+        let mut pty_master = pty
+            .master
+            .take_writer()
+            .map_err(|e| eyre!("Could not take PTY master writer: {e}"))?;
 
         // Spawn a git instance in the pty.
         let mut cmd = CommandBuilder::new(&git.path_to_git);
@@ -878,8 +966,18 @@ Screen contents:
                 }
 
                 PtyAction::Write(value) => {
-                    write!(pty.master, "{value}")?;
-                    pty.master.flush()?;
+                    if let Ok(Some(exit_status)) = child.try_wait() {
+                        panic!(
+                            "\
+Tried to write {value:?} to PTY, but the process has already exited with status {exit_status:?}. Screen contents:
+-----
+{}
+-----
+", parser.lock().unwrap().screen().contents(),
+                        );
+                    }
+                    write!(pty_master, "{value}")?;
+                    pty_master.flush()?;
                 }
             }
         }
@@ -893,7 +991,7 @@ Screen contents:
                 String::from_utf8(buffer).unwrap()
             }
         });
-        child.wait()?;
+        let exit_status = child.wait()?;
 
         let _ = read_remainder_of_pty_output_thread;
         // Useful for debugging, but seems to deadlock on some tests:
@@ -904,6 +1002,6 @@ Screen contents:
         //     console::strip_ansi_codes(&remainder_of_pty_output)
         // );
 
-        Ok(())
+        Ok(exit_status)
     }
 }
