@@ -15,7 +15,7 @@ use branchless::core::rewrite::{
     execute_rebase_plan, BuildRebasePlanOptions, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
     RebasePlan, RebasePlanBuilder, RepoResource,
 };
-use git_branchless_testing::{make_git, Git};
+use branchless::testing::{make_git, Git};
 
 #[test]
 fn test_cache_shared_between_builders() -> eyre::Result<()> {
@@ -632,34 +632,76 @@ fn test_plan_moving_subtree_with_merge_commit() -> eyre::Result<()> {
         Ok(())
     })?;
 
-    // FIXME: this is wrong, the merge commit should be moved as well.
     let stdout = git.smartlog()?;
+    insta::assert_snapshot!(stdout, @r###"
+    O f777ecc (master) create initial.txt
+    |
+    o 62fc20d create test1.txt
+    |
+    o 96d1c37 create test2.txt
+    |
+    o 70deb1e create test3.txt
+    |\
+    | o 355e173 create test4.txt
+    | & (merge) 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
+    |
+    @ 9ea1b36 create test5.txt
+    |
+    | & (merge) 355e173 create test4.txt
+    |/
+    o 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
+    "###);
+
+    Ok(())
+}
+
+#[test]
+fn test_plan_fixup_child_into_parent() -> eyre::Result<()> {
+    let git = make_git()?;
+    git.init_repo()?;
+    git.detach_head()?;
+    let _test1_oid = git.commit_file("test1", 1)?;
+    let test2_oid = git.commit_file("test2", 2)?;
+    let test3_oid = git.commit_file("test3", 3)?;
+
+    create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
+        builder.fixup_commit(test3_oid, test2_oid)?;
+        Ok(())
+    })?;
+
+    let (stdout, _stderr) = git.run(&["smartlog"])?;
     insta::assert_snapshot!(stdout, @r###"
         O f777ecc (master) create initial.txt
         |
         o 62fc20d create test1.txt
         |
-        o 96d1c37 create test2.txt
-        |\
-        | o b8f27a8 create test3.txt
-        | |\
-        | | @ 2b47b50 create test5.txt
-        | |
-        | o 22cf458 create test4.txt
+        @ 64da0f2 create test2.txt
+        "###);
+
+    Ok(())
+}
+
+#[test]
+fn test_plan_fixup_parent_into_child() -> eyre::Result<()> {
+    let git = make_git()?;
+    git.init_repo()?;
+    git.detach_head()?;
+    let _test1_oid = git.commit_file("test1", 1)?;
+    let test2_oid = git.commit_file("test2", 2)?;
+    let test3_oid = git.commit_file("test3", 3)?;
+
+    create_and_execute_plan(&git, move |builder: &mut RebasePlanBuilder| {
+        builder.fixup_commit(test2_oid, test3_oid)?;
+        Ok(())
+    })?;
+
+    let (stdout, _stderr) = git.run(&["smartlog"])?;
+    insta::assert_snapshot!(stdout, @r###"
+        O f777ecc (master) create initial.txt
         |
-        x 70deb1e (rewritten as b8f27a86) create test3.txt
-        |\
-        | x 355e173 (rewritten as 22cf4586) create test4.txt
-        | & (merge) 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
+        o 62fc20d create test1.txt
         |
-        x 9ea1b36 (rewritten as 2b47b505) create test5.txt
-        |
-        | & (merge) 355e173 (rewritten as 22cf4586) create test4.txt
-        |/
-        o 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
-        hint: there is 1 abandoned commit in your commit graph
-        hint: to fix this, run: git restack
-        hint: disable this hint by running: git config --global branchless.hint.smartlogFixAbandoned false
+        @ 1f599a2 create test3.txt
         "###);
 
     Ok(())
@@ -715,9 +757,9 @@ fn create_and_execute_plan(
         now,
         event_tx_id: event_log_db.make_transaction_id(now, "test plan")?,
         preserve_timestamps: false,
-        force_in_memory: true,
+        force_in_memory: false,
         force_on_disk: false,
-        resolve_merge_conflicts: false,
+        resolve_merge_conflicts: true,
         check_out_commit_options: CheckOutCommitOptions {
             additional_args: Default::default(),
             reset: false,
