@@ -19,7 +19,7 @@ use std::time::SystemTime;
 
 use eden_dag::VertexName;
 use lib::core::repo_ext::RepoExt;
-use lib::util::ExitCode;
+use lib::util::{ExitCode, EyreExitOr};
 use rayon::ThreadPoolBuilder;
 use tracing::instrument;
 
@@ -73,8 +73,9 @@ pub fn r#move(
     exacts: Vec<Revset>,
     resolve_revset_options: &ResolveRevsetOptions,
     move_options: &MoveOptions,
+    fixup: bool,
     insert: bool,
-) -> eyre::Result<ExitCode> {
+) -> EyreExitOr<()> {
     let sources_provided = !sources.is_empty();
     let bases_provided = !bases.is_empty();
     let exacts_provided = !exacts.is_empty();
@@ -90,7 +91,7 @@ pub fn r#move(
             Some(oid) => Revset(oid.to_string()),
             None => {
                 writeln!(effects.get_output_stream(), "No --dest argument was provided, and no OID for HEAD is available as a default")?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         },
     };
@@ -113,7 +114,7 @@ pub fn r#move(
             Ok(commit_sets) => union_all(&commit_sets),
             Err(err) => {
                 err.describe(effects)?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         };
     let base_oids: CommitSet =
@@ -121,7 +122,7 @@ pub fn r#move(
             Ok(commit_sets) => union_all(&commit_sets),
             Err(err) => {
                 err.describe(effects)?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         };
     let exact_components = match resolve_commits(
@@ -147,7 +148,7 @@ pub fn r#move(
                             dag.set_count(&component_roots)?,
                             component_roots
                         )?;
-                        return Ok(ExitCode(1));
+                        return Ok(Err(ExitCode(1)));
                     }
                 };
 
@@ -160,7 +161,7 @@ pub fn r#move(
                         dag.set_count(&component_parents)?,
                         component_parents
                     )?;
-                    return Ok(ExitCode(1));
+                    return Ok(Err(ExitCode(1)));
                 };
 
                 components.insert(component_root, component);
@@ -170,7 +171,7 @@ pub fn r#move(
         }
         Err(err) => {
             err.describe(effects)?;
-            return Ok(ExitCode(1));
+            return Ok(Err(ExitCode(1)));
         }
     };
 
@@ -191,12 +192,12 @@ pub fn r#move(
                     other.len(),
                     expr,
                 )?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         },
         Err(err) => {
             err.describe(effects)?;
-            return Ok(ExitCode(1));
+            return Ok(Err(ExitCode(1)));
         }
     };
 
@@ -205,7 +206,7 @@ pub fn r#move(
             Some(head_oid) => CommitSet::from(head_oid),
             None => {
                 writeln!(effects.get_output_stream(), "No --source or --base arguments were provided, and no OID for HEAD is available as a default")?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         }
     } else {
@@ -274,11 +275,11 @@ pub fn r#move(
             detect_duplicate_commits_via_patch_id,
         };
         let permissions = {
-            let commits_to_move = source_oids.clone();
+            let commits_to_move = &source_oids;
             let commits_to_move = commits_to_move.union(&union_all(
                 &exact_components.values().cloned().collect::<Vec<_>>(),
             ));
-            let commits_to_move = if insert {
+            let commits_to_move = if insert || fixup {
                 commits_to_move.union(&dag.query_children(CommitSet::from(dest_oid))?)
             } else {
                 commits_to_move
@@ -289,7 +290,7 @@ pub fn r#move(
                 Ok(permissions) => permissions,
                 Err(err) => {
                     err.describe(effects, &repo, &dag)?;
-                    return Ok(ExitCode(1));
+                    return Ok(Err(ExitCode(1)));
                 }
             }
         };
@@ -297,7 +298,15 @@ pub fn r#move(
 
         let source_roots = dag.query_roots(source_oids.clone())?;
         for source_root in dag.commit_set_to_vec(&source_roots)? {
-            builder.move_subtree(source_root, vec![dest_oid])?;
+            if fixup {
+                let commits = dag.query_descendants(CommitSet::from(source_root))?;
+                let commits = dag.commit_set_to_vec(&commits)?;
+                for commit in commits.iter() {
+                    builder.fixup_commit(*commit, dest_oid)?;
+                }
+            } else {
+                builder.move_subtree(source_root, vec![dest_oid])?;
+            }
         }
 
         let component_roots: CommitSet = exact_components.keys().cloned().collect();
@@ -353,7 +362,7 @@ pub fn r#move(
                                 "range of commits"
                             },
                         )?;
-                        return Ok(ExitCode(1));
+                        return Ok(Err(ExitCode(1)));
                     }
                 }
 
@@ -394,7 +403,14 @@ pub fn r#move(
                 }
             }
 
-            builder.move_subtree(component_root, vec![component_dest_oid])?;
+            if fixup {
+                let commits = dag.commit_set_to_vec(component)?;
+                for commit in commits.iter() {
+                    builder.fixup_commit(*commit, dest_oid)?;
+                }
+            } else {
+                builder.move_subtree(component_root, vec![component_dest_oid])?;
+            }
         }
 
         if insert {
@@ -414,7 +430,7 @@ pub fn r#move(
                                 component_roots[i - 1],
                                 component_roots[i]
                             )?;
-                            return Ok(ExitCode(1));
+                            return Ok(Err(ExitCode(1)));
                         }
                     }
 
@@ -434,7 +450,7 @@ pub fn r#move(
                             effects.get_output_stream(),
                             "The --insert flag cannot be used when moving subtrees or ranges with multiple heads."
                         )?;
-                        return Ok(ExitCode(1));
+                        return Ok(Err(ExitCode(1)));
                     }
                 }
             };
@@ -460,7 +476,7 @@ pub fn r#move(
     let result = match rebase_plan {
         Ok(None) => {
             writeln!(effects.get_output_stream(), "Nothing to do.")?;
-            return Ok(ExitCode(0));
+            return Ok(Ok(()));
         }
         Ok(Some(rebase_plan)) => {
             let options = ExecuteRebasePlanOptions {
@@ -483,18 +499,18 @@ pub fn r#move(
         }
         Err(err) => {
             err.describe(effects, &repo, &dag)?;
-            return Ok(ExitCode(1));
+            return Ok(Err(ExitCode(1)));
         }
     };
 
     match result {
-        ExecuteRebasePlanResult::Succeeded { rewritten_oids: _ } => Ok(ExitCode(0)),
+        ExecuteRebasePlanResult::Succeeded { rewritten_oids: _ } => Ok(Ok(())),
 
         ExecuteRebasePlanResult::DeclinedToMerge { failed_merge_info } => {
             failed_merge_info.describe(effects, &repo, MergeConflictRemediation::Retry)?;
-            Ok(ExitCode(1))
+            Ok(Err(ExitCode(1)))
         }
 
-        ExecuteRebasePlanResult::Failed { exit_code } => Ok(exit_code),
+        ExecuteRebasePlanResult::Failed { exit_code } => Ok(Err(exit_code)),
     }
 }
