@@ -9,26 +9,19 @@
 )]
 #![allow(clippy::too_many_arguments, clippy::blocks_in_if_conditions)]
 
-use clap::{Args, Command as ClapCommand, CommandFactory, Parser, ValueEnum};
-use lib::git::NonZeroOid;
-
 use std::ffi::OsString;
 use std::fmt::Display;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
+use clap::{Args, Command as ClapCommand, CommandFactory, Parser, ValueEnum};
+use lib::git::NonZeroOid;
+
 /// A revset expression. Can be a commit hash, branch name, or one of the
 /// various revset functions.
 #[derive(Clone, Debug)]
 pub struct Revset(pub String);
 
-impl Revset {
-    /// The default revset to render in the smartlog if no revset is provided by the user.
-    pub fn default_smartlog_revset() -> Self {
-        Self("((draft() | branches() | @) % main()) | branches() | @".to_string())
-    }
-}
-
 impl FromStr for Revset {
     type Err = std::convert::Infallible;
 
@@ -353,6 +346,20 @@ pub struct SmartlogArgs {
     pub resolve_revset_options: ResolveRevsetOptions,
 }
 
+/// The Git hosting provider to use, called a "forge".
+#[derive(Clone, Debug, ValueEnum)]
+pub enum ForgeKind {
+    /// Force-push branches to the default push remote.
+    Branch,
+
+    /// Force-push branches to the remote and create a pull request for each
+    /// branch using the `gh` command-line tool.
+    Github,
+
+    /// Submit code reviews to Phabricator using the `arc` command-line tool.
+    Phabricator,
+}
+
 /// Push commits to a remote.
 #[derive(Debug, Parser)]
 pub struct SubmitArgs {
@@ -365,6 +372,14 @@ pub struct SubmitArgs {
     #[clap(action, short = 'c', long = "create")]
     pub create: bool,
 
+    /// If the remote supports it, create code reviews in "draft" mode.
+    #[clap(action, short = 'd', long = "draft")]
+    pub draft: bool,
+
+    /// What kind of execution strategy to use for tools which need access to the working copy.
+    #[clap(short = 's', long = "strategy")]
+    pub strategy: Option<TestExecutionStrategy>,
+
     /// The commits to push. All branches attached to those commits will be
     /// pushed.
     #[clap(value_parser, default_value = "stack()")]
@@ -373,6 +388,14 @@ pub struct SubmitArgs {
     /// Options for resolving revset expressions.
     #[clap(flatten)]
     pub resolve_revset_options: ResolveRevsetOptions,
+
+    /// The Git hosting provider to use.
+    #[clap(short = 'F', long = "forge")]
+    pub forge: Option<ForgeKind>,
+
+    /// An optional message to include with the create or update operation.
+    #[clap(short = 'm', long = "message")]
+    pub message: Option<String>,
 }
 
 /// Run a command on each commit in a given set and aggregate the results.
@@ -403,6 +426,10 @@ pub enum Command {
     /// report.
     BugReport,
 
+    /// Use the partial commit selector UI as a Git-compatible difftool; see
+    /// git-difftool(1) for more information on Git difftools.
+    Difftool(scm_record::scm_diff_editor::Opts),
+
     /// Run internal garbage collection.
     Gc,
 
@@ -486,6 +513,10 @@ pub enum Command {
         #[clap(flatten)]
         move_options: MoveOptions,
 
+        /// Combine the moved commits and squash them into the destination commit.
+        #[clap(action, short = 'F', long = "fixup", conflicts_with = "insert")]
+        fixup: bool,
+
         /// Insert the subtree between the destination and it's children, if any.
         /// Only supported if the moved subtree has a single head.
         #[clap(action, short = 'I', long = "insert")]
@@ -528,7 +559,7 @@ pub enum Command {
     Restack {
         /// The IDs of the abandoned commits whose descendants should be
         /// restacked. If not provided, all abandoned commits are restacked.
-        #[clap(value_parser)]
+        #[clap(value_parser, default_value = "draft()")]
         revsets: Vec<Revset>,
 
         /// Options for resolving revset expressions.
