@@ -12,17 +12,10 @@
 use std::collections::HashSet;
 use std::ffi::OsString;
 use std::fmt::Write;
-use std::io;
 use std::time::SystemTime;
 
-use cursive::backends::crossterm;
-use cursive::CursiveRunnable;
-use cursive_buffered_backend::BufferedBackend;
-
 use git_branchless_invoke::CommandContext;
 use git_branchless_opts::RecordArgs;
-use git_record::Recorder;
-use git_record::{RecordError, RecordState};
 use itertools::Itertools;
 use lib::core::check_out::{check_out_commit, CheckOutCommitOptions};
 use lib::core::config::get_restack_preserve_timestamps;
@@ -38,16 +31,18 @@ use lib::core::rewrite::{
 };
 use lib::git::{
     process_diff_for_record, update_index, CategorizedReferenceName, FileMode, GitRunInfo,
-    NonZeroOid, Repo, ResolvedReferenceInfo, Stage, UpdateIndexCommand, WorkingCopyChangesType,
-    WorkingCopySnapshot,
+    MaybeZeroOid, NonZeroOid, Repo, ResolvedReferenceInfo, Stage, UpdateIndexCommand,
+    WorkingCopyChangesType, WorkingCopySnapshot,
 };
-use lib::util::ExitCode;
+use lib::try_exit_code;
+use lib::util::{ExitCode, EyreExitOr};
 use rayon::ThreadPoolBuilder;
-use tracing::instrument;
+use scm_record::{EventSource, RecordError, RecordState, Recorder, SelectedContents};
+use tracing::{instrument, warn};
 
 /// Commit changes in the working copy.
 #[instrument]
-pub fn command_main(ctx: CommandContext, args: RecordArgs) -> eyre::Result<ExitCode> {
+pub fn command_main(ctx: CommandContext, args: RecordArgs) -> EyreExitOr<()> {
     let CommandContext {
         effects,
         git_run_info,
@@ -79,7 +74,7 @@ fn record(
     branch_name: Option<String>,
     detach: bool,
     insert: bool,
-) -> eyre::Result<ExitCode> {
+) -> EyreExitOr<()> {
     let now = SystemTime::now();
     let repo = Repo::from_dir(&git_run_info.working_directory)?;
     let conn = repo.get_db_conn()?;
@@ -99,7 +94,7 @@ fn record(
                     effects.get_output_stream(),
                     "There are no changes to tracked files in the working copy to commit."
                 )?;
-                return Ok(ExitCode(0));
+                return Ok(Ok(()));
             }
             WorkingCopyChangesType::Unstaged | WorkingCopyChangesType::Staged => {}
             WorkingCopyChangesType::Conflicts => {
@@ -111,14 +106,14 @@ fn record(
                     effects.get_output_stream(),
                     "Resolve them and try again. Aborting."
                 )?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         }
         (snapshot, working_copy_changes_type)
     };
 
     if let Some(branch_name) = branch_name {
-        let exit_code = check_out_commit(
+        try_exit_code!(check_out_commit(
             effects,
             git_run_info,
             &repo,
@@ -130,13 +125,10 @@ fn record(
                 reset: false,
                 render_smartlog: false,
             },
-        )?;
-        if !exit_code.is_success() {
-            return Ok(exit_code);
-        }
+        )?);
     }
 
-    let commit_exit_code = if interactive {
+    if interactive {
         if working_copy_changes_type == WorkingCopyChangesType::Staged {
             writeln!(
                 effects.get_output_stream(),
@@ -146,16 +138,16 @@ fn record(
                 effects.get_output_stream(),
                 "Either commit or unstage your changes and try again. Aborting."
             )?;
-            ExitCode(1)
+            return Ok(Err(ExitCode(1)));
         } else {
-            record_interactive(
+            try_exit_code!(record_interactive(
                 effects,
                 git_run_info,
                 &repo,
                 &snapshot,
                 event_tx_id,
                 message.as_deref(),
-            )?
+            )?);
         }
     } else {
         let args = {
@@ -168,10 +160,7 @@ fn record(
             }
             args
         };
-        git_run_info.run_direct_no_wrapping(Some(event_tx_id), &args)?
-    };
-    if !commit_exit_code.is_success() {
-        return Ok(commit_exit_code);
+        try_exit_code!(git_run_info.run_direct_no_wrapping(Some(event_tx_id), &args)?);
     }
 
     if detach {
@@ -216,13 +205,15 @@ fn record(
     }
 
     if insert {
-        let exit_code = insert_before_siblings(effects, git_run_info, now, event_tx_id)?;
-        if !exit_code.is_success() {
-            return Ok(exit_code);
-        }
+        try_exit_code!(insert_before_siblings(
+            effects,
+            git_run_info,
+            now,
+            event_tx_id
+        )?);
     }
 
-    Ok(ExitCode(0))
+    Ok(Ok(()))
 }
 
 #[instrument]
@@ -233,11 +224,11 @@ fn record_interactive(
     snapshot: &WorkingCopySnapshot,
     event_tx_id: EventTransactionId,
     message: Option<&str>,
-) -> eyre::Result<ExitCode> {
-    let file_states = {
+) -> EyreExitOr<()> {
+    let old_tree = snapshot.commit_stage0.get_tree()?;
+    let new_tree = snapshot.commit_unstaged.get_tree()?;
+    let files = {
         let (effects, _progress) = effects.start_operation(OperationType::CalculateDiff);
-        let old_tree = snapshot.commit_stage0.get_tree()?;
-        let new_tree = snapshot.commit_unstaged.get_tree()?;
         let diff = repo.get_diff_between_trees(
             &effects,
             Some(&old_tree),
@@ -247,38 +238,93 @@ fn record_interactive(
         )?;
         process_diff_for_record(repo, &diff)?
     };
-    let record_state = RecordState { file_states };
+    let record_state = RecordState {
+        is_read_only: false,
+        files,
+    };
 
-    let siv = CursiveRunnable::new(|| -> io::Result<_> {
-        // Use crossterm to ensure that we support Windows.
-        let crossterm_backend = crossterm::Backend::init()?;
-        Ok(Box::new(BufferedBackend::new(crossterm_backend)))
-    });
-    let siv = siv.into_runner();
-
-    let recorder = Recorder::new(record_state);
-    let result = recorder.run(siv);
+    let recorder = Recorder::new(record_state, EventSource::Crossterm);
+    let result = recorder.run();
     let RecordState {
-        file_states: result,
+        is_read_only: _,
+        files: result,
     } = match result {
         Ok(result) => result,
         Err(RecordError::Cancelled) => {
             println!("Aborted.");
-            return Ok(ExitCode(1));
+            return Ok(Err(ExitCode(1)));
+        }
+        Err(RecordError::Bug(message)) => {
+            println!("BUG: {message}");
+            println!("This is a bug. Please report it.");
+            return Ok(Err(ExitCode(1)));
+        }
+        Err(
+            err @ (RecordError::SetUpTerminal(_)
+            | RecordError::CleanUpTerminal(_)
+            | RecordError::ReadInput(_)
+            | RecordError::RenderFrame(_)
+            | RecordError::SerializeJson(_)
+            | RecordError::WriteFile(_)),
+        ) => {
+            println!("Error: {err}");
+            return Ok(Err(ExitCode(1)));
         }
     };
 
     let update_index_script: Vec<UpdateIndexCommand> = result
         .into_iter()
-        .map(|(path, file_state)| -> eyre::Result<UpdateIndexCommand> {
-            let (selected, _unselected) = file_state.get_selected_contents();
-            let oid = repo.create_blob_from_contents(selected.as_bytes())?;
-            let command = UpdateIndexCommand::Update {
-                path,
-                stage: Stage::Stage0,
-                // TODO: use `FileMode::BlobExecutable` when appropriate.
-                mode: FileMode::Blob,
-                oid,
+        .map(|file| -> eyre::Result<UpdateIndexCommand> {
+            let mode = {
+                let default_mode = FileMode::Blob;
+                match file.get_file_mode() {
+                    None => {
+                        warn!(
+                            ?file,
+                            ?default_mode,
+                            "No file mode was set for file, using default"
+                        );
+                        default_mode
+                    }
+                    Some(mode) => match i32::try_from(mode) {
+                        Ok(mode) => FileMode::from(mode),
+                        Err(err) => {
+                            warn!(
+                                ?mode,
+                                ?default_mode,
+                                ?err,
+                                "File mode did not fit into i32, using default"
+                            );
+                            default_mode
+                        }
+                    },
+                }
+            };
+
+            let (selected, _unselected) = file.get_selected_contents();
+            let oid = match selected {
+                SelectedContents::Absent => MaybeZeroOid::Zero,
+                SelectedContents::Unchanged => {
+                    old_tree.get_oid_for_path(&file.path)?.unwrap_or_default()
+                }
+                SelectedContents::Binary {
+                    old_description: _,
+                    new_description: _,
+                } => new_tree.get_oid_for_path(&file.path)?.unwrap(),
+                SelectedContents::Present { contents } => {
+                    MaybeZeroOid::NonZero(repo.create_blob_from_contents(contents.as_bytes())?)
+                }
+            };
+            let command = match oid {
+                MaybeZeroOid::Zero => UpdateIndexCommand::Delete {
+                    path: file.path.clone().into_owned(),
+                },
+                MaybeZeroOid::NonZero(oid) => UpdateIndexCommand::Update {
+                    path: file.path.clone().into_owned(),
+                    stage: Stage::Stage0,
+                    mode,
+                    oid,
+                },
             };
             Ok(command)
         })
@@ -308,7 +354,7 @@ fn insert_before_siblings(
     git_run_info: &GitRunInfo,
     now: SystemTime,
     event_tx_id: EventTransactionId,
-) -> eyre::Result<ExitCode> {
+) -> EyreExitOr<()> {
     // Reopen the repository since references may have changed.
     let repo = Repo::from_dir(&git_run_info.working_directory)?;
     let conn = repo.get_db_conn()?;
@@ -326,7 +372,7 @@ fn insert_before_siblings(
             oid: None,
             reference_name: _,
         } => {
-            return Ok(ExitCode(0));
+            return Ok(Ok(()));
         }
     };
 
@@ -383,7 +429,7 @@ fn insert_before_siblings(
 
         Ok(None) => {
             // Nothing to do, since there were no siblings to move.
-            return Ok(ExitCode(0));
+            return Ok(Ok(()));
         }
 
         Err(BuildRebasePlanError::ConstraintCycle { .. }) => {
@@ -391,12 +437,12 @@ fn insert_before_siblings(
                 effects.get_output_stream(),
                 "BUG: constraint cycle detected when moving siblings, which shouldn't be possible."
             )?;
-            return Ok(ExitCode(1));
+            return Ok(Err(ExitCode(1)));
         }
 
         Err(err @ BuildRebasePlanError::MoveIllegalCommits { .. }) => {
             err.describe(effects, &repo, &dag)?;
-            return Ok(ExitCode(1));
+            return Ok(Err(ExitCode(1)));
         }
 
         Err(BuildRebasePlanError::MovePublicCommits {
@@ -423,7 +469,7 @@ To proceed anyways, run: git move -f -s 'siblings(.)",
                     .get_glyphs()
                     .render(example_bad_commit.friendly_describe(effects.get_glyphs())?)?,
             )?;
-            return Ok(ExitCode(0));
+            return Ok(Ok(()));
         }
     };
 
@@ -445,11 +491,11 @@ To proceed anyways, run: git move -f -s 'siblings(.)",
         &execute_options,
     )?;
     match result {
-        ExecuteRebasePlanResult::Succeeded { rewritten_oids: _ } => Ok(ExitCode(0)),
+        ExecuteRebasePlanResult::Succeeded { rewritten_oids: _ } => Ok(Ok(())),
         ExecuteRebasePlanResult::DeclinedToMerge { failed_merge_info } => {
             failed_merge_info.describe(effects, &repo, MergeConflictRemediation::Insert)?;
-            Ok(ExitCode(0))
+            Ok(Ok(()))
         }
-        ExecuteRebasePlanResult::Failed { exit_code } => Ok(exit_code),
+        ExecuteRebasePlanResult::Failed { exit_code } => Ok(Err(exit_code)),
     }
 }
