@@ -312,7 +312,7 @@ fn fn_message(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
         name,
         args,
         Box::new(move |_repo, commit| {
-            let message = commit.get_message_raw().map_err(PatternError::Repo)?;
+            let message = commit.get_message_raw();
             let message = match message.to_str() {
                 Ok(message) => message,
                 Err(err) => {
@@ -572,10 +572,12 @@ fn fn_tests_passed(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
                     let SerializedTestResult {
                         command,
                         exit_code,
-                        fixed_tree_oid: _,
+                        head_commit_oid: _,
+                        snapshot_tree_oid: _,
                         interactive: _,
                     } = test_result;
-                    exit_code == TEST_SUCCESS_EXIT_CODE && pattern.matches_text(&command)
+                    exit_code == TEST_SUCCESS_EXIT_CODE
+                        && pattern.matches_text(&command.to_string())
                 });
             Ok(result)
         }),
@@ -597,13 +599,14 @@ fn fn_tests_failed(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
                     let SerializedTestResult {
                         command,
                         exit_code,
-                        fixed_tree_oid: _,
+                        head_commit_oid: _,
+                        snapshot_tree_oid: _,
                         interactive: _,
                     } = test_result;
                     exit_code != TEST_SUCCESS_EXIT_CODE
                         && exit_code != TEST_INDETERMINATE_EXIT_CODE
                         && exit_code != TEST_ABORT_EXIT_CODE
-                        && pattern.matches_text(&command)
+                        && pattern.matches_text(&command.to_string())
                 });
             Ok(result)
         }),
@@ -625,16 +628,18 @@ fn fn_tests_fixable(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult
                     let SerializedTestResult {
                         command,
                         exit_code,
-                        fixed_tree_oid,
+                        head_commit_oid: _,
+                        snapshot_tree_oid,
                         interactive: _,
                     } = test_result;
                     exit_code == TEST_SUCCESS_EXIT_CODE
-                        && pattern.matches_text(&command)
-                        && match fixed_tree_oid {
-                            None => false,
-                            Some(SerializedNonZeroOid(fixed_tree_oid)) => {
-                                commit.get_tree_oid() != MaybeZeroOid::NonZero(fixed_tree_oid)
-                            }
+                        && pattern.matches_text(&command.to_string())
+                        && match (snapshot_tree_oid, commit.get_tree_oid()) {
+                            (
+                                Some(SerializedNonZeroOid(snapshot_tree_oid)),
+                                MaybeZeroOid::NonZero(original_tree_oid),
+                            ) => snapshot_tree_oid != original_tree_oid,
+                            (None, _) | (_, MaybeZeroOid::Zero) => false,
                         }
                 });
             Ok(result)
