@@ -350,12 +350,12 @@ pub(super) fn eval_number_rhs(
 mod tests {
     use std::borrow::Cow;
 
-    use git_branchless_testing::{make_git, GitRunOptions};
     use lib::core::effects::Effects;
     use lib::core::eventlog::{EventLogDb, EventReplayer};
     use lib::core::formatting::Glyphs;
     use lib::core::repo_ext::RepoExt;
     use lib::git::Commit;
+    use lib::testing::{make_git, GitRunOptions};
 
     use super::*;
     use crate::Expr;
@@ -372,7 +372,7 @@ mod tests {
             .into_iter()
             .map(|oid| repo.find_commit_or_fail(oid))
             .try_collect()?;
-        commits.sort_by_key(|commit| (commit.get_message_pretty().unwrap(), commit.get_time()));
+        commits.sort_by_key(|commit| (commit.get_message_pretty(), commit.get_time()));
         Ok(commits)
     }
 
