@@ -1,7 +1,8 @@
-use git_branchless_testing::{
-    make_git_with_remote_repo, GitInitOptions, GitRunOptions, GitWrapperWithRemoteRepo,
-};
 use lib::git::GitVersion;
+use lib::testing::{
+    make_git_with_remote_repo, remove_nondeterministic_lines, GitInitOptions, GitRunOptions,
+    GitWrapperWithRemoteRepo,
+};
 
 /// Minimum version due to changes in the output of `git push`.
 const MIN_VERSION: GitVersion = GitVersion(2, 36, 0);
@@ -12,6 +13,8 @@ fn redact_remotes(output: String) -> String {
         .map(|line| {
             if line.contains("To file://") {
                 "To: file://<remote>\n".to_string()
+            } else if line.contains("From file://") {
+                "From: file://<remote>\n".to_string()
             } else if line.contains("error: failed to push some refs to 'file://") {
                 "error: failed to push some refs to 'file://<remote>'\n".to_string()
             } else {
@@ -96,13 +99,16 @@ fn test_submit() -> eyre::Result<()> {
         let (stdout, stderr) = cloned_repo.run(&["submit"])?;
         let stderr = redact_remotes(stderr);
         insta::assert_snapshot!(stderr, @r###"
+        From: file://<remote>
+         * branch            bar        -> FETCH_HEAD
+         * branch            qux        -> FETCH_HEAD
         branchless: processing 1 update: branch qux
         To: file://<remote>
          + 20230db...bae8307 qux -> qux (forced update)
         branchless: processing 1 update: remote branch origin/qux
         "###);
         insta::assert_snapshot!(stdout, @r###"
-        branchless: running command: <git-executable> fetch origin
+        branchless: running command: <git-executable> fetch origin refs/heads/bar refs/heads/qux
         branchless: running command: <git-executable> push --force-with-lease origin qux
         Pushed 1 branch: qux
         Skipped 1 branch (already up-to-date): bar
@@ -113,9 +119,13 @@ fn test_submit() -> eyre::Result<()> {
     {
         let (stdout, stderr) = cloned_repo.run(&["submit", "--create"])?;
         let stderr = redact_remotes(stderr);
-        insta::assert_snapshot!(stderr, @"");
+        insta::assert_snapshot!(stderr, @r###"
+        From: file://<remote>
+         * branch            bar        -> FETCH_HEAD
+         * branch            qux        -> FETCH_HEAD
+        "###);
         insta::assert_snapshot!(stdout, @r###"
-        branchless: running command: <git-executable> fetch origin
+        branchless: running command: <git-executable> fetch origin refs/heads/bar refs/heads/qux
         Skipped 2 branches (already up-to-date): bar, qux
         "###);
     }
@@ -210,15 +220,11 @@ fn test_submit_existing_branch() -> eyre::Result<()> {
             },
         )?;
         let stderr = redact_remotes(stderr);
+        let stderr = remove_nondeterministic_lines(stderr);
         insta::assert_snapshot!(stderr, @r###"
         To: file://<remote>
          ! [rejected]        feature -> feature (fetch first)
         error: failed to push some refs to 'file://<remote>'
-        hint: Updates were rejected because the remote contains work that you do
-        hint: not have locally. This is usually caused by another repository pushing
-        hint: to the same ref. You may want to first integrate the remote changes
-        hint: (e.g., 'git pull ...') before pushing again.
-        hint: See the 'Note about fast-forwards' in 'git push --help' for details.
         "###);
         insta::assert_snapshot!(stdout, @"branchless: running command: <git-executable> push --set-upstream origin feature
 ");
@@ -285,9 +291,12 @@ fn test_submit_up_to_date_branch() -> eyre::Result<()> {
     {
         let (stdout, stderr) = cloned_repo.run(&["submit", "feature"])?;
         let stderr = redact_remotes(stderr);
-        insta::assert_snapshot!(stderr, @"");
+        insta::assert_snapshot!(stderr, @r###"
+        From: file://<remote>
+         * branch            feature    -> FETCH_HEAD
+        "###);
         insta::assert_snapshot!(stdout, @r###"
-        branchless: running command: <git-executable> fetch origin
+        branchless: running command: <git-executable> fetch origin refs/heads/feature
         Skipped 1 branch (already up-to-date): feature
         "###);
     }
