@@ -29,7 +29,8 @@ use cursive_core::{Cursive, CursiveRunner};
 use eyre::Context;
 use lib::core::check_out::{check_out_commit, CheckOutCommitOptions, CheckoutTarget};
 use lib::core::repo_ext::RepoExt;
-use lib::util::ExitCode;
+use lib::try_exit_code;
+use lib::util::{ExitCode, EyreExitOr};
 use tracing::instrument;
 
 use crate::tui::{with_siv, SingletonView};
@@ -780,7 +781,7 @@ fn undo_events(
     event_replayer: &EventReplayer,
     event_cursor: EventCursor,
     skip_confirmation: bool,
-) -> eyre::Result<ExitCode> {
+) -> EyreExitOr<()> {
     let now = SystemTime::now();
     let event_tx_id = event_log_db.make_transaction_id(now, "undo")?;
     let head_info = repo.get_head_info()?;
@@ -809,7 +810,7 @@ fn undo_events(
             effects.get_output_stream(),
             "No undo actions to apply, exiting."
         )?;
-        return Ok(ExitCode(0));
+        return Ok(Ok(()));
     }
     writeln!(effects.get_output_stream(), "Will apply these actions:")?;
     let events = describe_events_numbered(effects.get_glyphs(), repo, &inverse_events)?;
@@ -837,7 +838,7 @@ fn undo_events(
     };
     if !confirmed {
         writeln!(effects.get_output_stream(), "Aborted.")?;
-        return Ok(ExitCode(1));
+        return Ok(Err(ExitCode(1)));
     }
 
     let num_inverse_events = Pluralize {
@@ -916,7 +917,7 @@ fn undo_events(
     }
 
     if let Some(UndoCheckoutTarget { target, options }) = checkout_target {
-        let exit_code = check_out_commit(
+        try_exit_code!(check_out_commit(
             effects,
             git_run_info,
             repo,
@@ -925,14 +926,11 @@ fn undo_events(
             Some(target),
             &options,
         )
-        .wrap_err("Updating to previous HEAD location")?;
-        if !exit_code.is_success() {
-            return Ok(exit_code);
-        }
+        .wrap_err("Updating to previous HEAD location")?);
     }
 
     writeln!(effects.get_output_stream(), "Applied {num_inverse_events}.")?;
-    Ok(ExitCode(0))
+    Ok(Ok(()))
 }
 
 /// Restore the repository to a previous state interactively.
@@ -942,7 +940,7 @@ pub fn undo(
     git_run_info: &GitRunInfo,
     interactive: bool,
     skip_confirmation: bool,
-) -> eyre::Result<ExitCode> {
+) -> EyreExitOr<()> {
     let repo = Repo::from_current_dir()?;
     let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
@@ -969,7 +967,7 @@ pub fn undo(
             })?;
             match result {
                 Some(event_cursor) => event_cursor,
-                None => return Ok(ExitCode(0)),
+                None => return Ok(Ok(())),
             }
         } else {
             event_replayer.advance_cursor_by_transaction(event_replayer.make_default_cursor(), -1)
@@ -999,7 +997,7 @@ pub mod testing {
     use lib::core::effects::Effects;
     use lib::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
     use lib::git::{GitRunInfo, Repo};
-    use lib::util::ExitCode;
+    use lib::util::EyreExitOr;
 
     pub fn select_past_event(
         siv: CursiveRunner<Cursive>,
@@ -1019,7 +1017,7 @@ pub mod testing {
         event_log_db: &mut EventLogDb,
         event_replayer: &EventReplayer,
         event_cursor: EventCursor,
-    ) -> eyre::Result<ExitCode> {
+    ) -> EyreExitOr<()> {
         super::undo_events(
             in_,
             effects,
