@@ -4,7 +4,8 @@
 //! that are already tracked in the repo. Following the amend,
 //! the command performs a restack.
 
-use std::ffi::OsString;
+use std::collections::HashMap;
+
 use std::fmt::Write;
 use std::time::{SystemTime, UNIX_EPOCH};
 
@@ -22,14 +23,12 @@ use lib::core::formatting::Pluralize;
 use lib::core::gc::mark_commit_reachable;
 use lib::core::repo_ext::RepoExt;
 use lib::core::rewrite::{
-    execute_rebase_plan, BuildRebasePlanOptions, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
-    RebasePlanBuilder, RebasePlanPermissions, RepoResource,
+    execute_rebase_plan, move_branches, BuildRebasePlanOptions, ExecuteRebasePlanOptions,
+    ExecuteRebasePlanResult, RebasePlanBuilder, RebasePlanPermissions, RepoResource,
 };
-use lib::git::{
-    AmendFastOptions, CategorizedReferenceName, GitRunInfo, MaybeZeroOid, Repo,
-    ResolvedReferenceInfo,
-};
-use lib::util::ExitCode;
+use lib::git::{AmendFastOptions, GitRunInfo, MaybeZeroOid, Repo, ResolvedReferenceInfo};
+use lib::try_exit_code;
+use lib::util::{ExitCode, EyreExitOr};
 use rayon::ThreadPoolBuilder;
 use tracing::instrument;
 
@@ -41,7 +40,7 @@ pub fn amend(
     resolve_revset_options: &ResolveRevsetOptions,
     move_options: &MoveOptions,
     reparent: bool,
-) -> eyre::Result<ExitCode> {
+) -> EyreExitOr<()> {
     let now = SystemTime::now();
     let timestamp = now.duration_since(SystemTime::UNIX_EPOCH)?.as_secs_f64();
     let repo = Repo::from_current_dir()?;
@@ -66,7 +65,7 @@ pub fn amend(
                 effects.get_output_stream(),
                 "No commit is currently checked out. Check out a commit to amend and then try again.",
             )?;
-            return Ok(ExitCode(1));
+            return Ok(Err(ExitCode(1)));
         }
     };
     let head_commit = repo.find_commit_or_fail(head_oid)?;
@@ -77,7 +76,7 @@ pub fn amend(
             effects.get_output_stream(),
             "Cannot amend, because there are unresolved merge conflicts. Resolve the merge conflicts and try again."
         )?;
-        return Ok(ExitCode(1));
+        return Ok(Err(ExitCode(1)));
     }
 
     let build_options = BuildRebasePlanOptions {
@@ -92,7 +91,7 @@ pub fn amend(
         RebasePlanPermissions::verify_rewrite_set(&dag, build_options, &commits_to_verify)?
     {
         err.describe(effects, &repo, &dag)?;
-        return Ok(ExitCode(1));
+        return Ok(Err(ExitCode(1)));
     };
 
     let event_tx_id = event_log_db.make_transaction_id(now, "amend")?;
@@ -141,7 +140,7 @@ pub fn amend(
             effects.get_output_stream(),
             "There are no uncommitted or staged changes. Nothing to amend."
         )?;
-        return Ok(ExitCode(0));
+        return Ok(Ok(()));
     }
 
     let amended_tree = repo.amend_fast(&head_commit, &opts)?;
@@ -163,47 +162,51 @@ pub fn amend(
         None,
         Some(&amended_tree),
     )?;
-    mark_commit_reachable(&repo, amended_commit_oid)
-        .wrap_err("Marking commit as reachable for GC purposes.")?;
-    event_log_db.add_events(vec![Event::RewriteEvent {
-        timestamp: now.duration_since(UNIX_EPOCH)?.as_secs_f64(),
-        event_tx_id,
-        old_commit_oid: MaybeZeroOid::NonZero(head_oid),
-        new_commit_oid: MaybeZeroOid::NonZero(amended_commit_oid),
-    }])?;
-    dag.sync_from_oids(
-        effects,
-        &repo,
-        CommitSet::empty(),
-        CommitSet::from(amended_commit_oid),
-    )?;
-    let exit_code = {
-        let additional_args = match &head_info.reference_name {
-            Some(name) => match CategorizedReferenceName::new(name) {
-                name @ CategorizedReferenceName::LocalBranch { .. } => {
-                    vec![OsString::from("-B"), OsString::from(name.remove_prefix()?)]
-                }
-                CategorizedReferenceName::RemoteBranch { .. }
-                | CategorizedReferenceName::OtherRef { .. } => Default::default(),
-            },
-            None => Default::default(),
+
+    // Switch to the new commit and move any branches. This is kind of a hack:
+    // ideally, we would use the same rebase plan machinery to accomplish this
+    // and also rebase any descendants. However, this operation should always
+    // succeed, and we want to execute it regardless of whether the rest of the
+    // rebase would succeed without conflicts, so instead we (re)write a bunch
+    // of logic to switch commits and move branches.
+    {
+        mark_commit_reachable(&repo, amended_commit_oid)
+            .wrap_err("Marking commit as reachable for GC purposes.")?;
+        event_log_db.add_events(vec![Event::RewriteEvent {
+            timestamp: now.duration_since(UNIX_EPOCH)?.as_secs_f64(),
+            event_tx_id,
+            old_commit_oid: MaybeZeroOid::NonZero(head_oid),
+            new_commit_oid: MaybeZeroOid::NonZero(amended_commit_oid),
+        }])?;
+        dag.sync_from_oids(
+            effects,
+            &repo,
+            CommitSet::empty(),
+            CommitSet::from(amended_commit_oid),
+        )?;
+        move_branches(effects, git_run_info, &repo, event_tx_id, &{
+            let mut result = HashMap::new();
+            result.insert(head_oid, MaybeZeroOid::NonZero(amended_commit_oid));
+            result
+        })?;
+
+        let target = match &head_info.reference_name {
+            Some(name) => CheckoutTarget::Reference(name.clone()),
+            None => CheckoutTarget::Oid(amended_commit_oid),
         };
-        check_out_commit(
+        try_exit_code!(check_out_commit(
             effects,
             git_run_info,
             &repo,
             &event_log_db,
             event_tx_id,
-            Some(CheckoutTarget::Oid(amended_commit_oid)),
+            Some(target),
             &CheckOutCommitOptions {
-                additional_args,
+                additional_args: Default::default(),
                 reset: true,
                 render_smartlog: false,
             },
-        )?
-    };
-    if !exit_code.is_success() {
-        return Ok(exit_code);
+        )?);
     }
 
     let rebase_plan = {
@@ -226,7 +229,7 @@ pub fn amend(
             Ok(permissions) => permissions,
             Err(err) => {
                 err.describe(effects, &repo, &dag)?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         };
 
@@ -254,7 +257,7 @@ pub fn amend(
                     .into_iter()
                     .map(|parent_oid| repo.find_commit_or_fail(parent_oid))
                     .try_collect()?;
-                let descendant_message = descendant_commit.get_message_raw()?;
+                let descendant_message = descendant_commit.get_message_raw();
                 let descendant_message = descendant_message.to_str().with_context(|| {
                     eyre::eyre!(
                         "Could not decode commit message for descendant commit: {:?}",
@@ -279,7 +282,7 @@ pub fn amend(
             Ok(rebase_plan) => rebase_plan,
             Err(err) => {
                 err.describe(effects, &repo, &dag)?;
-                return Ok(ExitCode(1));
+                return Ok(Err(ExitCode(1)));
             }
         }
     };
@@ -340,7 +343,7 @@ pub fn amend(
             }
 
             ExecuteRebasePlanResult::Failed { exit_code } => {
-                return Ok(exit_code);
+                return Ok(Err(exit_code));
             }
         }
     }
@@ -370,7 +373,10 @@ pub fn amend(
                 "Amended with {uncommitted_changes}.",
             )?;
         }
+        AmendFastOptions::FromCommit { .. } => {
+            unreachable!("BUG: AmendFastOptions::FromCommit should not have been constructed.")
+        }
     }
 
-    Ok(ExitCode(0))
+    Ok(Ok(()))
 }
