@@ -1,10 +1,10 @@
 use eyre::{eyre, Context};
-use git_branchless_testing::make_git;
 use lib::core::effects::Effects;
 use lib::core::eventlog::testing::{get_event_replayer_events, redact_event_timestamp};
 use lib::core::eventlog::{Event, EventLogDb, EventReplayer};
 use lib::core::formatting::Glyphs;
 use lib::git::GitVersion;
+use lib::testing::make_git;
 use lib::util::get_sh;
 use std::process::Command;
 
@@ -107,9 +107,7 @@ fn test_fixup_no_abandoned_commit_message() -> eyre::Result<()> {
         if git_version < GitVersion(2, 35, 0) {
             insta::assert_snapshot!(stderr, @r###"
             branchless: processing 1 update: ref HEAD
-            branchless: processed commit: a84541d # This is a combination of 2 commits. # This is the 1st commit message:
             branchless: processing 1 update: ref HEAD
-            branchless: processed commit: 7f023a1 create test1.txt
             branchless: processing 3 rewritten commits
             Successfully rebased and updated detached HEAD.
             "###);
@@ -139,7 +137,6 @@ fn test_rebase_individual_commit() -> eyre::Result<()> {
         if git_version < GitVersion(2, 35, 0) {
             insta::assert_snapshot!(stderr, @r###"
             branchless: processing 1 update: ref HEAD
-            branchless: processed commit: f8d9985 create test2.txt
             branchless: processing 1 rewritten commit
             branchless: This operation abandoned 1 commit!
             branchless: Consider running one of the following:
