@@ -1,10 +1,10 @@
 use std::collections::HashMap;
 
 use eyre::Context;
-use git_branchless_testing::{
+use lib::git::GitVersion;
+use lib::testing::{
     make_git, make_git_worktree, GitInitOptions, GitRunOptions, GitWorktreeWrapper,
 };
-use lib::git::GitVersion;
 use regex::Regex;
 
 #[test]
@@ -281,7 +281,7 @@ fn test_init_prompt_for_main_branch() -> eyre::Result<()> {
 #[cfg(unix)]
 #[test]
 fn test_main_branch_not_found_error_message() -> eyre::Result<()> {
-    use git_branchless_testing::trim_lines;
+    use lib::testing::trim_lines;
 
     let git = make_git()?;
 
@@ -316,7 +316,7 @@ fn test_main_branch_not_found_error_message() -> eyre::Result<()> {
 
        0: branchless::core::eventlog::from_event_log_db with effects=<Output fancy=false> repo=<Git repository at: "<repo-path>/.git/"> event_log_db=<EventLogDb>
           at some/file/path.rs:123
-       1: git_branchless_smartlog::smartlog with effects=<Output fancy=false> git_run_info=<GitRunInfo path_to_git="<git-executable>" working_directory="<repo-path>" env=not shown> options=SmartlogOptions { event_id: None, revset: Revset("((draft() | branches() | @) % main()) | branches() | @"), resolve_revset_options: ResolveRevsetOptions { show_hidden_commits: false }, reverse: false }
+       1: git_branchless_smartlog::smartlog with effects=<Output fancy=false> git_run_info=<GitRunInfo path_to_git="<git-executable>" working_directory="<repo-path>" env=not shown> options=SmartlogOptions { event_id: None, revset: None, resolve_revset_options: ResolveRevsetOptions { show_hidden_commits: false }, reverse: false }
           at some/file/path.rs:123
        2: git_branchless_smartlog::command_main with ctx=CommandContext { effects: <Output fancy=false>, git_run_info: <GitRunInfo path_to_git="<git-executable>" working_directory="<repo-path>" env=not shown> } args=SmartlogArgs { event_id: None, revset: None, reverse: false, resolve_revset_options: ResolveRevsetOptions { show_hidden_commits: false } }
           at some/file/path.rs:123
@@ -651,7 +651,7 @@ fn test_install_man_pages() -> eyre::Result<()> {
     insta::assert_snapshot!(man_page_contents, @r###"
     .ie \n(.g .ds Aq \(aq
     .el .ds Aq '
-    .TH git-branchless 1  "git-branchless 0.7.1" 
+    .TH git-branchless 1  "git-branchless 0.8.0" 
     .SH NAME
     git\-branchless \- Branchless workflow for Git
     .SH SYNOPSIS
@@ -681,10 +681,10 @@ fn test_install_man_pages() -> eyre::Result<()> {
     .RE
     .TP
     \fB\-h\fR, \fB\-\-help\fR
-    Print help information (use `\-h` for a summary)
+    Print help (see a summary with \*(Aq\-h\*(Aq)
     .TP
     \fB\-V\fR, \fB\-\-version\fR
-    Print version information
+    Print version
     .SH SUBCOMMANDS
     .TP
     git\-branchless\-amend(1)
@@ -693,6 +693,9 @@ fn test_install_man_pages() -> eyre::Result<()> {
     git\-branchless\-bug\-report(1)
     Gather information about recent operations to upload as part of a bug report
     .TP
+    git\-branchless\-difftool(1)
+    Use the partial commit selector UI as a Git\-compatible difftool; see git\-difftool(1) for more information on Git difftools
+    .TP
     git\-branchless\-gc(1)
     Run internal garbage collection
     .TP
@@ -756,7 +759,7 @@ fn test_install_man_pages() -> eyre::Result<()> {
     git\-branchless\-help(1)
     Print this message or the help of the given subcommand(s)
     .SH VERSION
-    v0.7.1
+    v0.8.0
     .SH AUTHORS
     Waleed Khan <me@waleedkhan.name>
     "###);
