@@ -2,15 +2,17 @@ use std::mem::swap;
 use std::rc::Rc;
 use std::sync::{Arc, Mutex};
 
-use git_branchless_testing::{make_git, trim_lines, Git, GitInitOptions};
 use git_branchless_undo::testing::{select_past_event, undo_events};
-use git_record::testing::{screen_to_string, CursiveTestingBackend, CursiveTestingEvent};
+use git_branchless_undo::tui::testing::{
+    screen_to_string, CursiveTestingBackend, CursiveTestingEvent,
+};
 use lib::core::dag::Dag;
 use lib::core::effects::Effects;
 use lib::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
 use lib::core::formatting::Glyphs;
 use lib::core::repo_ext::RepoExt;
 use lib::git::{GitRunInfo, GitVersion, Repo};
+use lib::testing::{make_git, trim_lines, Git, GitInitOptions, GitRunOptions};
 
 use cursive_core::event::Key;
 use cursive_core::{Cursive, CursiveRunner};
@@ -58,7 +60,7 @@ fn run_undo_events(git: &Git, event_cursor: EventCursor) -> eyre::Result<(isize,
         env: git.get_base_env(0).into_iter().collect(),
     };
 
-    let ExitCode(exit_code) = undo_events(
+    let exit_code = match undo_events(
         &mut in_,
         &Effects::new_from_buffer_for_test(glyphs, &stdout, &stderr),
         &repo,
@@ -66,7 +68,10 @@ fn run_undo_events(git: &Git, event_cursor: EventCursor) -> eyre::Result<(isize,
         &mut event_log_db,
         &event_replayer,
         event_cursor,
-    )?;
+    )? {
+        Ok(()) => 0,
+        Err(ExitCode(exit_code)) => exit_code,
+    };
 
     let stdout = {
         let mut buf = stdout.lock().unwrap();
@@ -817,7 +822,7 @@ fn test_undo_noninteractive() -> eyre::Result<()> {
     {
         let (stdout, _stderr) = git.run_with_options(
             &["undo"],
-            &git_branchless_testing::GitRunOptions {
+            &GitRunOptions {
                 expected_exit_code: 1,
                 input: Some("n".to_string()),
                 ..Default::default()
@@ -849,7 +854,7 @@ fn test_undo_noninteractive() -> eyre::Result<()> {
     {
         let (stdout, _stderr) = git.run_with_options(
             &["undo"],
-            &git_branchless_testing::GitRunOptions {
+            &GitRunOptions {
                 input: Some("y".to_string()),
                 ..Default::default()
             },
