@@ -14,7 +14,13 @@ mod types;
 mod ui;
 mod util;
 
+pub mod consts;
+pub mod helpers;
 pub use types::{
     ChangeType, File, FileMode, RecordError, RecordState, Section, SectionChangedLine,
+    SelectedContents,
 };
-pub use ui::Recorder;
+pub use ui::{Event, EventSource, Recorder, TestingScreenshot};
+
+#[cfg(feature = "scm-diff-editor")]
+pub mod scm_diff_editor;
